# -*- mode: perl; coding: utf-8 -*-
# Keitairc::View
# $Id: View.pm,v 1.14.2.1 2008-07-20 02:22:32 morimoto Exp $
# $Source: /home/ishikawa/work/keitairc/tmp/keitairc/lib/Keitairc/View.pm,v $
#
# Copyright (c) 2008 Jun Morimoto <morimoto@mrmt.net>
# This program is covered by the GNU General Public License 2

package Keitairc::View;
use HTML::Template;
use HTTP::Response;
use Keitairc::Config;
use Keitairc::ClientInfo;
use strict;
use warnings;

################################################################
sub new{
	my $proto = shift;
	my $cf = shift;
	my $ci = shift;
	my $session_id = shift;
	my $me = {};
	$me->{Config} = $cf;
	$me->{ClientInfo} = $ci;
	if(defined $session_id){
		$me->{session_id} = $session_id;
	}
        bless $me;
}

################################################################
sub redirect{
	my $me = shift;
	my $path = shift;

	if($me->{Config}->web_root() ne '/'){
		# Softbank
		$path =~ s|^/||;
		$path = sprintf('%s%s', $me->{Config}->web_root(), $path);
	}

	::log_debug("path [$path]");

	my $response = HTTP::Response->new(302);
	$response->push_header('Location', $path);
	$response;
}

################################################################
sub template_path{
	my $me = shift;
	my $template = shift;
	for my $template_dir (split(':', $me->{Config}->template_dir())){
		if(-r "$template_dir/$template"){
			return "$template_dir/$template";
		}
	}
}

################################################################
sub public_path{
	my $me = shift;
	my $file = shift;
	for my $public_dir (split(':', $me->{Config}->public_dir())){
		if(-r "$public_dir/$file"){
			return "$public_dir/$file";
		}
	}
}

################################################################
sub template{
	my $me = shift;
	my $template = shift;
	my $vars = shift;

	my $p = {};
	$p->{root} = $me->{Config}->web_root();
	$p->{title} = $me->{Config}->web_title();
	$p->{version} = $me->{Config}->version();
	$p->{use_iui} = $me->{Config}->use_iui();

	if($me->{ClientInfo}->is_mobile){
		$p->{fontsize} = $me->{Config}->mobile_fontsize();
	}else{
		$p->{fontsize} = $me->{Config}->fontsize();
	}
	$p->{timestamp} = timestamp();

	$p->{is_ipod} = $me->{ClientInfo}->is_ipod();
	$p->{is_docomo} = $me->{ClientInfo}->is_docomo();
	$p->{is_ezweb} = $me->{ClientInfo}->is_ezweb();
	$p->{is_softbank} = $me->{ClientInfo}->is_softbank();
	$p->{is_emobile} = $me->{ClientInfo}->is_emobile();
	$p->{is_mobile} = $me->{ClientInfo}->is_mobile();

	map {
		$p->{"plugin_$_"} = 1;
	} $::pl->list_plugins();

	my $arch = 'chtml-sjis';
	if($me->{ClientInfo}->is_ipod()){
		$arch = 'ipod-sjis';
	}

	my $t = new HTML::Template(
		filename => $me->template_path("$arch/$template"),
		die_on_bad_params => 0,
		cache => 1);
	$t->param($p);

	if(defined $vars){
		$t->param($vars);
	}

	$t->output();
}

################################################################
sub public{
	my $me = shift;
	my $file = shift;
	my $mime_types = {
		html => 'text/html',
		css => 'text/css',
		gif => 'image/gif',
		png => 'image/png',
		jpg => 'image/jpg',
		js => 'application/javascript',
	};

	if(open(my $fh, '<', $me->public_path($file))){
		local $/;
		undef $/;
		my $buf = join('', <$fh>);
		close($fh);

		my ($ext) = ($file =~ /\.(.*)$/);
		my $mime_type = $mime_types->{$ext} || 'text/plain';

		my $response = HTTP::Response->new(200);
		$response->push_header('Content-type', $mime_type);
		$response->content($buf);
		return $response;
	}

	# not found
	my $response = HTTP::Response->new(404);
	$response->push_header('Content-type', 'text/html; charset=Shift_JIS');
	$response->content($me->template('404.html', { action => $file }));
	return $response;
}

################################################################
sub render{
	my $me = shift;
	my $template = shift;
	my $vars = shift;
	my $response = HTTP::Response->new(200);
	$vars->{content_type} |= 'text/html; charset=Shift_JIS';
	$response->push_header('Content-type', $vars->{content_type});
	$response->content($me->template($template, $vars));
	$response;
}

################################################################
sub timestamp{
	my ($sec, $min, $hour, $day, $mon, $year) = localtime(time);
	sprintf('%d/%d/%d %02d:%02d:%02d',
		$year+1900, $mon+1, $day, $hour, $min, $sec);
}

1;
